@@ -68,8 +68,10 @@ class Articut:
         # Toolkit
         self.analyse = AnalyseManager()
         self.localRE = TaiwanAddressAnalizer(locale="TW")
-        self.NER = GenericNER()
         self.LawsToolkit = LawsToolkit()
+        self.NER = GenericNER()
+        self.POS = ArticutPOS()
+
         #try:
             #self.graphQL = GraphQL()
         #except:
@@ -78,34 +80,6 @@ class Articut:
     def __str__(self):
         return "Articut API"
 
-    def _segIndexConverter(self, parseResultDICT, posIndexLIST):
-        '''
-        Convert posIndex to segIndex
-        Return list
-        '''
-
-        if type(posIndexLIST) is list and "result_pos" in parseResultDICT:
-            pass
-        else:
-            return None
-
-        segIndexLIST = []
-        try:
-            for i, posLIST in enumerate(posIndexLIST):
-                if posLIST:
-                    tmpLIST = []
-                    for start, end, seg in posLIST:
-                        posEndSTR = parseResultDICT["result_pos"][i][:start]
-                        segEndSTR = "".join([x.group() for x in self.stripPat.finditer(posEndSTR)])
-                        tmpLIST.append((len(segEndSTR), len(segEndSTR)+len(seg), seg))
-                    segIndexLIST.append(tmpLIST)
-                else:
-                    segIndexLIST.append(posLIST)
-        except Exception:
-            print("Invalid posIndexLIST format")
-            return None
-        return segIndexLIST
-
     def parse(self, inputSTR, level="", userDefinedDictFILE=None, chemicalBOOL=True, openDataPlaceAccessBOOL=False, wikiDataBOOL=False, indexWithPOS=False, timeRef=None, pinyin="BOPOMOFO"):
         if level not in ("lv1", "lv2", "lv3"):
             level = self.level
@@ -176,62 +150,74 @@ class Articut:
     ##############################################################################
     #                                 Toolkits                                   #
     ##############################################################################
-    def getPersonLIST(self, parseResultDICT, includePronounBOOL=True, indexWithPOS=True):
+    def getAddTWLIST(self, parseResultDICT, indexWithPOS=True):
         '''
-        取出斷詞結果中的人名 (Person)
-        若 includePronounBOOL 為 True，則連代名詞 (Pronoun) 一併回傳；若為 False，則只回傳人名。
-        回傳結果為一個 list。
+        取出斷詞結果中含有 (KNOWLEDGE_addTW) 標籤的字串。
+        該字串為一台灣地址。
         '''
-        return getPersonLIST(parseResultDICT, includePronounBOOL, indexWithPOS)
-
-    def getContentWordLIST(self, parseResultDICT, indexWithPOS=True):
-        '''
-        取出斷詞結果中的實詞 (content word)。
-        每個句子內的實詞為一個 list。
-        '''
-        return getContentWordLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getAddTWLIST(parseResultDICT, indexWithPOS)
 
     def getChemicalLIST(self, parseResultDICT, indexWithPOS=True):
         '''
         取出斷詞結果中的化學類詞 (KNOWLEDGE_chemical)。
         每個句子內的化學類詞為一個 list。
         '''
-        return getChemicalLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getChemicalLIST(parseResultDICT, indexWithPOS)
 
-    def getVerbStemLIST(self, parseResultDICT, indexWithPOS=True):
+    def getColorLIST(self, resultDICT, indexWithPOS=True):
         '''
-        取出斷詞結果中的動詞 (verb)。此處指的是 ACTION_verb 標記的動詞詞彙。
-        每個句子內的動詞為一個 list。
+        取出斷詞結果中含有 (MODIFIER_color) 標籤的字串。
+        該字串為一顏色表述字串。
         '''
-        return getVerbStemLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getColorLIST(resultDICT, indexWithPOS)
 
-    def getNounStemLIST(self, parseResultDICT, indexWithPOS=True):
+    def getContentWordLIST(self, parseResultDICT, indexWithPOS=True):
         '''
-        取出斷詞結果中的名詞 (noun)。此處指的是 ENTITY_noun、ENTITY_nouny、ENTITY_nounHead 或 ENTITY_oov 標記的名詞詞彙。
-        每個句子內的名詞為一個 list。
+        取出斷詞結果中的實詞 (content word)。
+        每個句子內的實詞為一個 list。
         '''
-        return getNounStemLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getContentWordLIST(parseResultDICT, indexWithPOS)
 
-    def getTimeLIST(self, parseResultDICT, indexWithPOS=True):
+    def getCurrencyLIST(self, parseResultDICT, indexWithPOS=True, greedyBOOL=False):
         '''
-        取出斷詞結果中的時間 (time)。
-        每個句子內的「時間」詞列為一個 list。
+        取出斷詞結果中的貨幣金額 (KNOWLEDGE_currency) 標籤的字串。
+        每個句子內的「貨幣金額」，將列為一個 list。
+        若 greedy = True，則以下格式會加到回傳 list
+            貨幣名稱 + 數字 (包含「'」與「,」符號)
+            新台幣 100
+            美金9.99
+            歐元 1,999'99
         '''
-        return getTimeLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getCurrencyLIST(parseResultDICT, indexWithPOS, greedyBOOL)
 
     def getLocationStemLIST(self, parseResultDICT, indexWithPOS=True):
         '''
         取出斷詞結果中的地理位置 (location)。此處指的是地理位置標記的行政區地名詞彙，例如「台北」、「桃園」、「墨西哥」。
         每個句子內的地理位置列為一個 list。
         '''
-        return getLocationStemLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getLocationStemLIST(parseResultDICT, indexWithPOS)
+
+    def getNounStemLIST(self, parseResultDICT, indexWithPOS=True):
+        '''
+        取出斷詞結果中的名詞 (noun)。此處指的是 ENTITY_noun、ENTITY_nouny、ENTITY_nounHead 或 ENTITY_oov 標記的名詞詞彙。
+        每個句子內的名詞為一個 list。
+        '''
+        return self.POS.getNounStemLIST(parseResultDICT, indexWithPOS)
 
     def getOpenDataPlaceLIST(self, parseResultDICT, indexWithPOS=True):
         '''
         取出斷詞結果中的景點 (KNOWLEDGE_place) 標籤的字串。此處指的是景點 (KNOWLEDGE_place)標記的非行政地點名稱詞彙，例如「鹿港老街」、「宜蘭運動公園」。
         每個句子內的景點為一個 list.
         '''
-        return getOpenDataPlaceLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getOpenDataPlaceLIST(parseResultDICT, indexWithPOS)
+
+    def getPersonLIST(self, parseResultDICT, includePronounBOOL=True, indexWithPOS=True):
+        '''
+        取出斷詞結果中的人名 (Person)
+        若 includePronounBOOL 為 True，則連代名詞 (Pronoun) 一併回傳；若為 False，則只回傳人名。
+        回傳結果為一個 list。
+        '''
+        return self.POS.getPersonLIST(parseResultDICT, includePronounBOOL, indexWithPOS)
 
     def getQuestionLIST(self, parseResultDICT, indexWithPOS=True):
         '''
@@ -247,34 +233,28 @@ class Articut:
             <CLAUSE_WhyQ>: 「原因」問句
         每個句子內若有 <CLAUSE_Q> 標籤，整個句子將會存進 list。
         '''
-        return getQuestionLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getQuestionLIST(parseResultDICT, indexWithPOS)
 
-    def getAddTWLIST(self, parseResultDICT, indexWithPOS=True):
+    def getTimeLIST(self, parseResultDICT, indexWithPOS=True):
         '''
-        取出斷詞結果中含有 (KNOWLEDGE_addTW) 標籤的字串。
-        該字串為一台灣地址。
+        取出斷詞結果中的時間 (time)。
+        每個句子內的「時間」詞列為一個 list。
         '''
-        return getAddTWLIST(parseResultDICT, indexWithPOS)
+        return self.POS.getTimeLIST(parseResultDICT, indexWithPOS)
 
-    def getCurrencyLIST(self, parseResultDICT, indexWithPOS=True, greedyBOOL=False):
+    def getVerbStemLIST(self, parseResultDICT, indexWithPOS=True):
         '''
-        取出斷詞結果中的貨幣金額 (KNOWLEDGE_currency) 標籤的字串。
-        每個句子內的「貨幣金額」，將列為一個 list。
-        若 greedy = True，則以下格式會加到回傳 list
-            貨幣名稱 + 數字 (包含「'」與「,」符號)
-            新台幣 100
-            美金9.99
-            歐元 1,999'99
+        取出斷詞結果中的動詞 (verb)。此處指的是 ACTION_verb 標記的動詞詞彙。
+        每個句子內的動詞為一個 list。
         '''
-        return getCurrencyLIST(parseResultDICT, indexWithPOS, greedyBOOL)
+        return self.POS.getVerbStemLIST(parseResultDICT, indexWithPOS)
 
     def getWikiDataLIST(self, parseResultDICT, indexWithPOS=True):
         '''
         取出斷詞結果中的 WikiData 標記文字。此處指的是 KNOWLEDGE_wikiData 標記的條目名稱。
         每個句子內的條目名稱為一個 list。
         '''
-        return getWikiDataLIST(parseResultDICT, indexWithPOS)
-
+        return self.POS.getWikiDataLIST(parseResultDICT, indexWithPOS)
 
 
 if __name__ == "__main__":
@@ -297,30 +277,30 @@ if __name__ == "__main__":
     resultExistBOOL = False
     try:
         with open(resultFilePath, "r", encoding="utf-8") as resultFile:
-            result = json.loads(resultFile.read())
+            resultDICT = json.loads(resultFile.read())
             # inputSTR 去除空白及斜線
             # result_segmentation 去除斜線
-            if inputSTR.replace(' ', '').replace("/", "") == result["result_segmentation"].replace("/", ""):
+            if inputSTR.replace(' ', '').replace("/", "") == resultDICT["result_segmentation"].replace("/", ""):
                 resultExistBOOL = True
     except:
         pass
 
     #取得斷詞結果
     if not resultExistBOOL:
-        result = articut.parse(inputSTR, level="lv2", openDataPlaceAccessBOOL=False, wikiDataBOOL=False)
+        resultDICT = articut.parse(inputSTR, level="lv2", openDataPlaceAccessBOOL=False, wikiDataBOOL=False)
 
         #儲存斷詞結果
         try:
             with open(resultFilePath, "w", encoding="utf-8") as resultFile:
-                json.dump(result, resultFile, ensure_ascii=False)
+                json.dump(resultDICT, resultFile, ensure_ascii=False)
                 print("斷詞結果儲存成功")
         except Exception as e:
             print("斷詞結果儲存失敗：{}".format(e))
 
     print("\n斷詞結果：")
-    pprint(result["result_segmentation"])
+    pprint(resultDICT["result_segmentation"])
     print("\n標記結果：")
-    pprint(result["result_pos"])
+    pprint(resultDICT["result_pos"])
 
     #列出目前可使用的 Articut 版本選擇。通常版本號愈大，完成度愈高。
     versions = articut.versions()
@@ -328,128 +308,128 @@ if __name__ == "__main__":
     pprint(versions)
 
     #列出所有的 content word.
-    contentWordLIST = articut.getContentWordLIST(result)
+    contentWordLIST = articut.getContentWordLIST(resultDICT)
     print("\n##ContentWord:")
     pprint(contentWordLIST)
 
     #列出所有的化學類名詞.
-    chemicalLIST = articut.getChemicalLIST(result)
+    chemicalLIST = articut.getChemicalLIST(resultDICT)
     print("\n##Chemical:")
     pprint(chemicalLIST)
 
     #列出所有的人名 (不含代名詞).
-    personLIST = articut.getPersonLIST(result, includePronounBOOL=False)
+    personLIST = articut.getPersonLIST(resultDICT, includePronounBOOL=False)
     print("\n##Person (Without Pronoun):")
     pprint(personLIST)
-    personLIST = articut.getPersonLIST(result, includePronounBOOL=True)
+    personLIST = articut.getPersonLIST(resultDICT, includePronounBOOL=True)
     print("\n##Person (With Pronoun):")
     pprint(personLIST)
 
     #列出所有的 verb word. (動詞)
-    verbStemLIST = articut.getVerbStemLIST(result)
+    verbStemLIST = articut.getVerbStemLIST(resultDICT)
     print("\n##Verb:")
     pprint(verbStemLIST)
 
     #列出所有的 noun word. (名詞)
-    nounStemLIST = articut.getNounStemLIST(result)
+    nounStemLIST = articut.getNounStemLIST(resultDICT)
     print("\n##Noun:")
     pprint(nounStemLIST)
 
     #列出所有的 time (時間)
-    timeLIST = articut.getTimeLIST(result)
+    timeLIST = articut.getTimeLIST(resultDICT)
     print("\n##Time:")
     pprint(timeLIST)
 
     #列出所有的 location word. (地方名稱)
-    locationStemLIST = articut.getLocationStemLIST(result)
+    locationStemLIST = articut.getLocationStemLIST(resultDICT)
     print("\n##Location:")
     pprint(locationStemLIST)
 
     #允許 Articut 調用字典，列出所有政府開放資料中列為觀光地點名稱的字串。(地點名稱)
-    placeLIST = articut.getOpenDataPlaceLIST(result)
+    placeLIST = articut.getOpenDataPlaceLIST(resultDICT)
     print("\n##Place:")
     pprint(placeLIST)
 
     #允許 Articut 調用 WikiData 字典，列出所有 WikiData 條目名稱的字串。
-    wikiDataLIST = articut.getWikiDataLIST(result)
+    wikiDataLIST = articut.getWikiDataLIST(resultDICT)
     print("\n##WikiData:")
     pprint(wikiDataLIST)
 
     #列出所有的 CLAUSE 問句
-    questionLIST = articut.getQuestionLIST(result)
+    questionLIST = articut.getQuestionLIST(resultDICT)
     print("\n##Question:")
     pprint(questionLIST)
 
     #列出所有的台灣地址
-    addTWLIST = articut.getAddTWLIST(result)
+    addTWLIST = articut.getAddTWLIST(resultDICT)
     print("\n##Address:")
     pprint(addTWLIST)
 
     #使用 TF-IDF 演算法
-    tfidfResult = articut.analyse.extract_tags(result)
+    tfidfResult = articut.analyse.extract_tags(resultDICT)
     print("\n##TF-IDF:")
     pprint(tfidfResult)
 
     #使用 Textrank 演算法
-    textrankResult = articut.analyse.textrank(result)
+    textrankResult = articut.analyse.textrank(resultDICT)
     print("\n##Textrank:")
     pprint(textrankResult)
 
     #使用 localRE 工具取得地址分段細節
-    countyResult = articut.localRE.getAddressCounty(result)
+    countyResult = articut.localRE.getAddressCounty(resultDICT)
     print("\n##localRE: 縣")
     pprint(countyResult)
 
-    cityResult = articut.localRE.getAddressCity(result)
+    cityResult = articut.localRE.getAddressCity(resultDICT)
     print("\n##localRE: 市")
     pprint(cityResult)
 
-    districtResult = articut.localRE.getAddressDistrict(result)
+    districtResult = articut.localRE.getAddressDistrict(resultDICT)
     print("\n##localRE: 區")
     pprint(districtResult)
 
-    townshipResult = articut.localRE.getAddressTownship(result)
+    townshipResult = articut.localRE.getAddressTownship(resultDICT)
     print("\n##localRE: 鄉里")
     pprint(townshipResult)
 
-    townResult = articut.localRE.getAddressTown(result)
+    townResult = articut.localRE.getAddressTown(resultDICT)
     print("\n##localRE: 鎮")
     pprint(townResult)
 
-    villageResult = articut.localRE.getAddressVillage(result)
+    villageResult = articut.localRE.getAddressVillage(resultDICT)
     print("\n##localRE: 村")
     pprint(villageResult)
 
-    neighborhoodResult = articut.localRE.getAddressNeighborhood(result)
+    neighborhoodResult = articut.localRE.getAddressNeighborhood(resultDICT)
     print("\n##localRE: 鄰")
     pprint(neighborhoodResult)
 
-    roadResult = articut.localRE.getAddressRoad(result)
+    roadResult = articut.localRE.getAddressRoad(resultDICT)
     print("\n##localRE: 路")
     pprint(roadResult)
 
-    sectionResult = articut.localRE.getAddressSection(result)
+    sectionResult = articut.localRE.getAddressSection(resultDICT)
     print("\n##localRE: 段")
     pprint(sectionResult)
 
-    alleyResult = articut.localRE.getAddressAlley(result)
+    alleyResult = articut.localRE.getAddressAlley(resultDICT)
     print("\n##localRE: 巷、弄")
     pprint(alleyResult)
 
-    numberResult = articut.localRE.getAddressNumber(result)
+    numberResult = articut.localRE.getAddressNumber(resultDICT)
     print("\n##localRE: 號")
     pprint(numberResult)
 
-    floorResult = articut.localRE.getAddressFloor(result)
+    floorResult = articut.localRE.getAddressFloor(resultDICT)
     print("\n##localRE: 樓")
     pprint(floorResult)
 
-    roomResult = articut.localRE.getAddressRoom(result)
+    roomResult = articut.localRE.getAddressRoom(resultDICT)
     print("\n##localRE: 室")
     pprint(roomResult)
 
     #列出所有的貨幣金額
-    currencyResult = articut.getCurrencyLIST(result)
+    currencyResult = articut.getCurrencyLIST(resultDICT)
     print("\n##currencyLIST:")
     pprint(currencyResult)
 
